@@ -518,6 +518,16 @@ static void compile_block(jit_gencode_t gen, jit_function_t func,
 			}
 			break;
 
+			case JIT_OP_MARK_OFFSET:
+			{
+				/* Mark the current code position as corresponding
+				   to a particular bytecode offset */
+				_jit_cache_mark_bytecode
+					(&(gen->posn), (unsigned long)(long)
+							jit_value_get_nint_constant(insn->value1));
+			}
+			break;
+
 			default:
 			{
 				/* Generate code for the instruction with the back end */
@@ -690,6 +700,20 @@ int jit_function_compile(jit_function_t func)
 
 		/* End the function's output process */
 		result = _jit_cache_end_method(&(gen.posn));
+
+		/* If we need to restart on a different cache page, then clear
+		   the block addresses and fixup lists */
+		if(result == JIT_CACHE_END_RESTART)
+		{
+			block = 0;
+			while((block = jit_block_next(func, block)) != 0)
+			{
+				block->address = 0;
+				block->fixup_list = 0;
+				block->fixup_absolute_list = 0;
+			}
+		}
+
 	}
 	while(result == JIT_CACHE_END_RESTART);
 
@@ -1172,7 +1196,7 @@ int jit_function_apply_vararg
 	if(setjmp(jbuf.buf))
 	{
 		_jit_unwind_pop_setjmp();
-		return 1;
+		return 0;
 	}
 
 	/* Create a backtrace entry that blocks exceptions from
