@@ -402,8 +402,11 @@ jit_opcode_info_t const jit_opcodes[JIT_OP_NUM_OPCODES] = {
 	{"call",						JIT_OPCODE_IS_CALL},
 	{"call_tail",					JIT_OPCODE_IS_CALL},
 	{"call_indirect",				F_(EMPTY, PTR, EMPTY) | INDIRECT_ARGS},
+	{"call_indirect_tail",			F_(EMPTY, PTR, EMPTY) | INDIRECT_ARGS},
 	{"call_vtable_ptr",				F_(EMPTY, PTR, EMPTY)},
+	{"call_vtable_ptr_tail",		F_(EMPTY, PTR, EMPTY)},
 	{"call_external",				JIT_OPCODE_IS_CALL_EXTERNAL},
+	{"call_external_tail",			JIT_OPCODE_IS_CALL_EXTERNAL},
 	{"return",						F_(EMPTY, EMPTY, EMPTY)},
 	{"return_int",					F_(EMPTY, INT, EMPTY)},
 	{"return_long",					F_(EMPTY, LONG, EMPTY)},
@@ -470,6 +473,7 @@ jit_opcode_info_t const jit_opcodes[JIT_OP_NUM_OPCODES] = {
 	{"set_param_float64",			F_(EMPTY, FLOAT64, PTR)},
 	{"set_param_nfloat",			F_(EMPTY, NFLOAT, PTR)},
 	{"set_param_struct",			F_(PTR, PTR, PTR)},
+	{"push_return_area_ptr",		F_(EMPTY, EMPTY, EMPTY)},
 
 	/*
 	 * Pointer-relative loads and stores.
@@ -526,6 +530,12 @@ jit_opcode_info_t const jit_opcodes[JIT_OP_NUM_OPCODES] = {
 	 */
 	{"alloca",						F_(PTR, PTR, EMPTY)},
 
+	/*
+	 * Debugging support.
+	 */
+	{"mark_offset",					F_(EMPTY, INT, EMPTY)},
+	{"mark_breakpoint",				F_(EMPTY, PTR, PTR)},
+
 };
 
 #if defined(JIT_BACKEND_INTERP)
@@ -595,7 +605,6 @@ jit_opcode_info_t const _jit_interp_opcodes[JIT_OP_NUM_INTERP_OPCODES] = {
 	{"push_return_float64",			0},
 	{"push_return_nfloat",			0},
 	{"push_return_small_struct",	JIT_OPCODE_NINT_ARG},
-	{"push_return_area_ptr",		0},
 
 	/*
 	 * Nested function call handling.
