@@ -439,6 +439,15 @@ int _jit_create_call_setup_insns
 				return 0;
 			}
 		}
+		else if((flags & JIT_CALL_NATIVE) != 0)
+		{
+			/* Native calls always return a return area pointer */
+			if(!jit_insn_push_return_area_ptr(func))
+			{
+				return 0;
+			}
+			*struct_return = 0;
+		}
 		else
 		{
 			*struct_return = 0;
@@ -1094,6 +1103,47 @@ void _jit_gen_insn(jit_gencode_t gen, jit_function_t func,
 		}
 		/* Not reached */
 
+		case JIT_OP_JUMP_TABLE:
+		{
+			jit_label_t *labels;
+			jit_nint num_labels;
+			jit_nint index;
+
+			labels = (jit_label_t *) insn->value1->address;
+			num_labels = insn->value2->address;
+
+			_jit_regs_spill_all(gen);
+			_jit_regs_load_to_top(gen,
+					      insn->dest,
+					      (insn->flags & (JIT_INSN_DEST_NEXT_USE |
+							      JIT_INSN_DEST_LIVE)),
+					      0);
+
+			jit_cache_opcode(&(gen->posn), insn->opcode);
+			jit_cache_native(&(gen->posn), num_labels);
+			for(index = 0; index < num_labels; index++)
+			{
+				block = jit_block_from_label(func, labels[index]);
+				if(!block)
+				{
+					return;
+				}
+				if(block->address)
+				{
+					/* We already know the address of the block */
+					jit_cache_native(&(gen->posn), block->address);
+				}
+				else
+				{
+					/* Record this position on the block's fixup list */
+					pc = (void **)(gen->posn.ptr);
+					jit_cache_native(&(gen->posn), block->fixup_absolute_list);
+					block->fixup_absolute_list = pc;
+				}
+			}
+		}
+		break;
+
 		case JIT_OP_ADDRESS_OF_LABEL:
 		{
 			/* Get the address of a particular label */
@@ -1136,41 +1186,30 @@ void _jit_gen_insn(jit_gencode_t gen, jit_function_t func,
 		break;
 
 		case JIT_OP_CALL_INDIRECT:
+		case JIT_OP_CALL_INDIRECT_TAIL:
 		{
 			/* Call a function, whose pointer is supplied on the stack */
-			if(!jit_type_return_via_pointer
-					(jit_type_get_return((jit_type_t)(insn->value2))))
-			{
-				jit_cache_opcode(&(gen->posn), JIT_OP_PUSH_RETURN_AREA_PTR);
-			}
-			reg = _jit_regs_load_to_top(gen, insn->value1, 0, 0);
 			jit_cache_opcode(&(gen->posn), insn->opcode);
 			jit_cache_native(&(gen->posn), (jit_nint)(insn->value2));
 			jit_cache_native(&(gen->posn), (jit_nint)
 					(jit_type_num_params((jit_type_t)(insn->value2))));
-			_jit_regs_free_reg(gen, reg, 1);
 			adjust_working(gen, -1);
 		}
 		break;
 
 		case JIT_OP_CALL_VTABLE_PTR:
+		case JIT_OP_CALL_VTABLE_PTR_TAIL:
 		{
 			/* Call a function, whose vtable pointer is supplied on the stack */
-			reg = _jit_regs_load_to_top(gen, insn->value1, 0, 0);
 			jit_cache_opcode(&(gen->posn), insn->opcode);
-			_jit_regs_free_reg(gen, reg, 1);
 			adjust_working(gen, -1);
 		}
 		break;
 
 		case JIT_OP_CALL_EXTERNAL:
+		case JIT_OP_CALL_EXTERNAL_TAIL:
 		{
 			/* Call a native function, whose pointer is supplied explicitly */
-			if(!jit_type_return_via_pointer
-					(jit_type_get_return((jit_type_t)(insn->value2))))
-			{
-				jit_cache_opcode(&(gen->posn), JIT_OP_PUSH_RETURN_AREA_PTR);
-			}
 			jit_cache_opcode(&(gen->posn), insn->opcode);
 			jit_cache_native(&(gen->posn), (jit_nint)(insn->value2));
 			jit_cache_native(&(gen->posn), (jit_nint)(insn->dest));
@@ -1475,6 +1514,15 @@ void _jit_gen_insn(jit_gencode_t gen, jit_function_t func,
 		}
 		break;
 
+		case JIT_OP_PUSH_RETURN_AREA_PTR:
+		{
+			/* Push the address of the interpreter's return area */
+			_jit_regs_spill_all(gen);
+			jit_cache_opcode(&(gen->posn), insn->opcode);
+			adjust_working(gen, 1);
+		}
+		break;
+
 		case JIT_OP_POP_STACK:
 		{
 			/* Pop parameter values from the stack after a function returns */
@@ -1627,6 +1675,15 @@ void _jit_gen_insn(jit_gencode_t gen, jit_function_t func,
 		}
 		break;
 
+		case JIT_OP_MARK_BREAKPOINT:
+		{
+			/* Mark the current location as a potential breakpoint */
+			jit_cache_opcode(&(gen->posn), insn->opcode);
+			jit_cache_native(&(gen->posn), insn->value1->address);
+			jit_cache_native(&(gen->posn), insn->value2->address);
+		}
+		break;
+
 		default:
 		{
 			/* Whatever opcodes are left are ordinary operators,
@@ -1735,6 +1792,15 @@ void _jit_gen_start_block(jit_gencode_t gen, jit_block_t block)
 	}
 	block->fixup_list = 0;
 
+	fixup = (void **)(block->fixup_absolute_list);
+	while(fixup != 0)
+	{
+		next = (void **)(fixup[0]);
+		fixup[0] = (void *)(jit_nint)((void **)(block->address));
+		fixup = next;
+	}
+	block->fixup_absolute_list = 0;
+
 	/* If this is the exception catcher block, then we need to update
 	   the exception cookie for the function to point to here */
 	if(block->label == block->func->builder->catcher_label &&
