@@ -648,6 +648,7 @@ jit_value jit_function::get_struct_pointer()
  * @deftypemethodx jit_function void insn_push_ptr ({const jit_value&} value, jit_type_t type)
  * @deftypemethodx jit_function void insn_set_param ({const jit_value&} value, jit_nint offset)
  * @deftypemethodx jit_function void insn_set_param_ptr ({const jit_value&} value, jit_type_t type, jit_nint offset)
+ * @deftypemethodx jit_function void insn_push_return_area_ptr ()
  * @deftypemethodx jit_function void insn_return ({const jit_value&} value)
  * @deftypemethodx jit_function void insn_return ()
  * @deftypemethodx jit_function void insn_return_ptr ({const jit_value&} value, jit_type_t type)
@@ -671,6 +672,8 @@ jit_value jit_function::get_struct_pointer()
  * @deftypemethodx jit_function jit_value jit_insn_alloca ({const jit_value&} size)
  * @deftypemethodx jit_function void insn_move_blocks_to_end ({const jit_label&} from_label, {const jit_label&} to_label)
  * @deftypemethodx jit_function void insn_move_blocks_to_start ({const jit_label&} from_label, {const jit_label&} to_label)
+ * @deftypemethodx jit_function void insn_mark_offset (jit_int offset)
+ * @deftypemethodx jit_function void insn_mark_breakpoint (jit_nint data1, jit_nint data2)
  * Create instructions of various kinds.  @xref{Instructions}, for more
  * information on the individual instructions and their arguments.
  * @end deftypemethod
@@ -1242,6 +1245,14 @@ void jit_function::insn_set_param_ptr
 	}
 }
 
+void jit_function::insn_push_return_area_ptr()
+{
+	if(!jit_insn_push_return_area_ptr(func))
+	{
+		out_of_memory();
+	}
+}
+
 void jit_function::insn_return(const jit_value& value)
 {
 	if(!jit_insn_return(func, value.raw()))
@@ -1417,6 +1428,22 @@ void jit_function::insn_move_blocks_to_start
 	}
 }
 
+void jit_function::insn_mark_offset(jit_int offset)
+{
+	if(!jit_insn_mark_offset(func, offset))
+	{
+		out_of_memory();
+	}
+}
+
+void jit_function::insn_mark_breakpoint(jit_nint data1, jit_nint data2)
+{
+	if(!jit_insn_mark_breakpoint(func, data1, data2))
+	{
+		out_of_memory();
+	}
+}
+
 void jit_function::register_on_demand()
 {
 	jit_function_set_on_demand_compiler(func, on_demand_compiler);
